library("ath1121501.db")
library("topGO")
library("plyr")

find.missing <- function(x.file, y.file,
                         x.name, y.name){
    if(x.file != y.file){
        t1 = read.table(x.file, as.is = T, header = F, skip = 3)
        t2 = read.table(y.file, as.is = T, header = F, skip = 3)
        mnames = setdiff(t1$V1, t2$V1)
        missing = data.frame(missing = mnames)
        missing$Id = probe_ids2[mnames, "id"]
        missing$Alias = probe_ids2[mnames, "alias"]
        missing$Present = x.name
        missing$Absent = y.name
        missing
    }
}


write.missing <- function(x.file, y.file,
                          x.name, y.name,
                          dir.out = "../data/missing/"){
    if(x.file != y.file){
        missing = find.missing(x.file, y.file, x.name, y.name)
        missing = missing[, c(1,2,3)]
        write.table(missing, paste(dir.out, "present.in.", x.name, "-",
                                   "not.in", y.name, ".csv", sep = ""), sep = ",",
                    row.names = F);
    }
}

write.missing.allpairs <- function(gene.file.names, gene.names){
    for(x in 1:length(gene.file.names)){
        for(y in 1:length(gene.file.names)){
            x.file = gene.file.names[x]
            y.file = gene.file.names[y]
            x.tiss = gene.names[x]
            y.tiss = gene.names[y]
            write.missing(x.file,y.file, x.tiss, y.tiss)
        }
    }
}

missing.allpairs <- function(gene.file.names, gene.names){
    ldply(1:length(gene.file.names), function(x){
        ldply(1:length(gene.file.names), function(y){
            x.file = gene.file.names[x]
            y.file = gene.file.names[y]
            x.tiss = gene.names[x]
            y.tiss = gene.names[y]
            missing.ids = find.missing(x.file, y.file, x.tiss, y.tiss)$Id
            nids = length(missing.ids)
            if(nids > 0){
                data.frame(Present = rep(x.tiss, n), Absent = rep(y.tiss, n),
                           Id = missing.ids)
            } else {
                NA
            }
        })
    })
}

gen.names.cmd <- function(data.dir){
    bname = function(x){unlist(str_split(x, "-"))[6]}
    cnames = unlist(llply(dir(data.dir, "*.csv"), bname))
    for(x in dir(data.dir, "*.csv")){
        cat_cmd = paste("cat ",  x,  " | cut -f 1 > ",
            paste(bname(x), ".names", sep = ""))
        print(cat_cmd)
    }
}

get.probes <- function(probe.file){
    probe_ids <- read.table(probe.file, sep = "\t", quote = "\"",
                        col.names = c("ath", "id", "alias"),
                        colClasses = rep("character",3))
    row.names(probe_ids) = probe_ids$ath
    probe_ids
}


print.gnames <- function(){
    for(x in c(2,4,5)){
        for(y in c(2,4,5)){
            x.file = tiss.gene.fnames[x]
            y.file = tiss.gene.fnames[y]
            x.tiss = tiss.names[x]
            y.tiss = tiss.names[y]
            missing.ids = find.missing(x.file, y.file, x.tiss, y.tiss)$Id
            if(length(missing.ids) > 0){
                print(paste(x.tiss, y.tiss,
                            sum(leaf.ids %in% missing.ids),
                            sum(flower.ids %in% missing.ids),
                            sum(root.ids %in% missing.ids)))
            }
        }
    }
}

run.top.GO <- function(iqr.vector, go.desc,
                       go.ont = "BP",
                       iqr.limit = 1.0){
    topIQR <- function(x){x > iqr.limit};
    new("topGOdata", description = go.desc, ontology = go.ont,
        allGenes = iqr.vector, geneSel = topIQR,
        nodeSize = 50, annot = annFUN.db, affyLib = "ath1121501.db")
}

run.fisher <- function(tgo.data){
    runTest(tgo.data, algorithm = "classic", statistic = "fisher")
}

run.ks <- function(tgo.data) {
    runTest(tgo.data, algorithm = "classic", statistic = "ks")
}

run.elim.ks <- function(tgo.data) {
    runTest(tgo.data, algorithm = "elim", statistic = "ks")
}

top.GO.tests <- function(iqr.vector, go.desc,
                          go.ont = "BP",
                          iqr.limit = 0.8){
    tgo.data <- run.top.GO(iqr.vector, go.desc,
                           go.ont = go.ont, iqr.limit = iqr.limit)
    fisher.results <- run.fisher(tgo.data)
    ks.results <- run.ks(tgo.data)
    elim.ks.results <- run.elim.ks(tgo.data)
    avg.results <- combineResults(fisher.results, ks.results, elim.ks.results)
    list(tgo.data, fisher.results, ks.results, elim.ks.results, avg.results)
}

fisher.top.GO <- function(iqr.vector, go.desc,
                          go.ont = "BP",
                          iqr.limit = 0.8,
                          ntop = 20){
    tgo.data <- run.top.GO(iqr.vector, go.desc,
                           go.ont = go.ont, iqr.limit = iqr.limit)
    fisher.results <- run.fisher(tgo.data)
    ks.results <- run.ks(tgo.data)
    elim.ks.results <- run.elim.ks(tgo.data)
    GenTable(tgo.data, classicFisher = fisher.results,
             classicKS = ks.results, elimKS = elim.ks.results,
             orderBy = "elimKS", ranksOf="classicFisher", topNodes = ntop)
}

common.names <- function(iqr.vector.x, iqr.vector.y, iqr.limit = 0.7){
    union(names(iqr.vector.x[iqr.vector.x > iqr.limit]),
          names(iqr.vector.y[iqr.vector.y > iqr.limit]))
}

common.names.lst <- function(iqr.vec.lst, iqr.limit = 0.7){
    ldply(iqr.vec.lst, function(iqr.vector.x){
        data.frame(probes = as.character(names(
                       iqr.vector.x[iqr.vector.x > iqr.limit])))
    })
}

common.names.lst2 <- function(iqr.vec.lst, iqr.limit = 0.7){
    nlen <- length(iqr.vec.lst)
    ndf <- ldply(1:nlen, function(i){
        iqr.vector.x <- iqr.vec.lst[[i]]
        cnames <- as.character(names(
            iqr.vector.x[iqr.vector.x > iqr.limit]))
        cdf <- data.frame(probes = cnames, iqr = iqr.vector.x[cnames])
        cdf$lst <- i
        cdf
    })
    ndf
    ddply(ndf, .(probes), function(x){
         if(nrow(x) == 1){
             c(iqr = x[1, "iqr"], lst = x[1, "lst"]);
         }
    })
}


fisher.top.common.GO <- function(iqr.vector.x, go.desc.x,
                                 iqr.vector.y, go.desc.y,
                                 go.ont = "BP",
                                 iqr.limit.n = 0.7,
                                 iqr.limit.g = 0.8,
                                 ntop = 20){

    cnames <- common.names(iqr.vector.x, iqr.vector.y,
                           iqr.limit = iqr.limit.n)
    list(fisher.top.GO(iqr.vector.x[cnames], go.desc.x,
                       go.ont = go.ont, iqr.limit = iqr.limit.g,
                       ntop = ntop),
         fisher.top.GO(iqr.vector.y[cnames], go.desc.y,
                       go.ont = go.ont, iqr.limit = iqr.limit.g,
                       ntop = ntop))
}

top.common.lst.GO <- function(iqr.vec.lst, go.desc.x,
                              go.ont = "BP",
                              iqr.limit.n = 0.7,
                              iqr.limit.g = 0.8){
    cnames <- levels(common.names.lst(
        iqr.vec.lst, iqr.limit = iqr.limit.n)$probes)
    run.top.GO(iqr.vec.lst[[1]][cnames], go.desc.x,
               go.ont = go.ont, iqr.limit = iqr.limit.g)
}

top.common.lst.GO.tests <- function(iqr.vec.lst, go.desc.x,
                                     go.ont = "BP",
                                     iqr.limit.n = 1.0,
                                     iqr.limit.g = 1.0){
    cnames <- levels(common.names.lst(
        iqr.vec.lst, iqr.limit = iqr.limit.n)$probes)
    top.GO.tests(iqr.vec.lst[[1]][cnames], go.desc.x,
                 go.ont = go.ont, iqr.limit = iqr.limit.g)
}


fisher.top.common.lst.GO <- function(iqr.vec.lst, go.desc.x,
                                     go.ont = "BP",
                                     iqr.limit.n = 0.7,
                                     iqr.limit.g = 0.8,
                                     ntop = 20){
    cnames <- levels(common.names.lst(
        iqr.vec.lst, iqr.limit = iqr.limit.n)$probes)
    fisher.top.GO(iqr.vec.lst[[1]][cnames], go.desc.x,
                  go.ont = go.ont, iqr.limit = iqr.limit.g,
                  ntop = ntop)
}


top.common.lst2.GO <- function(iqr.vec.lst, go.desc.x,
                               go.ont = "BP",
                               iqr.limit.n = 1.0,
                               iqr.limit.g = 1.0){
    cnames <- as.character(common.names.lst2(
        iqr.vec.lst, iqr.limit = iqr.limit.n)$probes)
    run.top.GO(iqr.vec.lst[[1]][cnames], go.desc.x,
               go.ont = go.ont, iqr.limit = iqr.limit.g)
}


fisher.top.common.lst2.GO <- function(iqr.vec.lst, go.desc.x,
                                      go.ont = "BP",
                                      iqr.limit.n = 1.0,
                                      iqr.limit.g = 1.0,
                                      ntop = 50){
    cnames <- as.character(common.names.lst2(
        iqr.vec.lst, iqr.limit = iqr.limit.n)$probes)
    fisher.top.GO(iqr.vec.lst[[1]][cnames], go.desc.x,
                  go.ont = go.ont, iqr.limit = iqr.limit.g,
                  ntop = ntop)
}

top.common.lst2.GO.tests <- function(iqr.vec.lst, go.desc.x,
                                     go.ont = "BP",
                                     iqr.limit.n = 1.0,
                                     iqr.limit.g = 1.0){
    cnames <- as.character(common.names.lst2(
        iqr.vec.lst, iqr.limit = iqr.limit.n)$probes)
    top.GO.tests(iqr.vec.lst[[1]][cnames], go.desc.x,
                 go.ont = go.ont, iqr.limit = iqr.limit.g)
}


sel.genes.df <- function(x.data, y.data,
                         x.name, y.name,
                         sel.genes) {
  xdata.sel <- x.data[ sel.genes, ]
  ydata.sel <- y.data[ sel.genes, ]
  xdata.sel <- as.data.frame(t(xdata.sel))
  ydata.sel <- as.data.frame(t(ydata.sel))
  colnames(xdata.sel) <- paste("X", colnames(xdata.sel),  sep = "")
  colnames(ydata.sel) <- paste("X", colnames(ydata.sel),  sep = "")
  xdata.sel$When <- x.name
  ydata.sel$When <- y.name
  rbind(xdata.sel, ydata.sel)
}



sel.genes.df2 <- function(data.lst,
                          data.names,
                          sel.genes) {
    cnames <-  paste("X", sel.genes, sep = "")
    tdf <- ldply(1:length(data.names), function(j){
        x.df <- (data.lst[[j]])[ sel.genes, ]
        x.df <- as.data.frame(t(x.df))
        ## print(nrow(x.df))
        print(colnames(x.df))
        ## colnames(x.df) <- cnames;
        x.df$When <- data.names[j];
        x.df
    })
    colnames(tdf) <- c(cnames, "When");
    tdf
}

sel.genes.plot <- function(comp.df,
                           gene.name,
                           gene.column = gene.name,
                           x.limits = c(-1.5,1.5),
                           l.pos = c(0.9, 0.9),
                           font.name = "Arial",
                           font.size = 12){
    ggplot(comp.df, aes_string(x = gene.name, colour = "When")) +
    stat_density( binwidth = 0.1, position = "identity", geom = "line" , size = 1.2) +
    scale_x_continuous(limits = x.limits) +
    ylab("Kernel Density of Expression") +
    xlab(paste("Expression Value for probe ", gene.column)) +
    labs(fill = "",colour = "") +
    theme_bw() +
    theme(text=element_text(family=font.name, size=font.size),
          legend.position = l.pos)
}


gen.iqr.range.plots <- function(data.lst, data.names, genes.lst,
                                out.dir, ...){
    cdf2 = sel.genes.df2(data.lst, data.names, genes.lst)
    for(xgn in genes.lst){
        gname = paste("X", xgn, sep = "")
        plt2 = sel.genes.plot(cdf2, gname, xgn, ...);
        fname = paste(out.dir, "/", xgn, ".pdf", sep = "")
        ggsave(filename = fname, plt2)
    }
}


sel_genes_df <- function(jqdata, eqdata, sel_genes) {
  edata_sel <- eqdata[ sel_genes, ]
  jdata_sel <- jqdata[ sel_genes, ]
  edata_sel <- as.data.frame(t(edata_sel))
  jdata_sel <- as.data.frame(t(jdata_sel))
  colnames(edata_sel) <- paste("X", colnames(edata_sel),  sep = "")
  colnames(jdata_sel) <- paste("X", colnames(jdata_sel),  sep = "")
  jdata_sel$When <- "2010"
  edata_sel$When <- "2014"
  rbind(jdata_sel, edata_sel)
}

sel_genes_mat <- function(matx, maty, sx, sy, sel_genes){
    m1 = matx[sel_genes,]
    m2 = maty[sel_genes,]
    mcmb = cbind(m1, m2)
    mdf = as.data.frame(t(mcmb))
    colnames(mdf) = paste("X", colnames(mdf), sep="")
    mdf$class = "NULL"
    mdf$class[1:dim(m1)[2]] = sx
    mdf$class[(1 + dim(m1)[2]):(dim(m1)[2] +  dim(m2)[2])] = sy
    mdf
}
