source("preprocess.R")
source("io_env.R")


move_small_size_dir <- function(final_data_df, db_limit,
                                dry_run = T){
  ddply( final_data_df, .( db, id ), function( x ) {
    nfiles <- nrow(x)
    db <- x[1,1]
    id <- x[1,2]
    if(nfiles < db_limit){
      mv_command <- paste("mv",
                          paste(data.final.dir, db, id, sep = "/"),
                          paste(data.final.tmp.dir, id, sep = "/"))
      ret_val <- 0
      if(!dry_run) {
        ret_val <- system(mv_command)
      }
      c(count = nfiles, rv = ret_val)
    } else {
      c(count = nfiles, rv = -1)
    }
  })
}

run_mas5 <- function(in_dir, out_prefix,
                     in_files = character(0)){
  in_files <- if(length(in_files) == 0) {
    dir(in_dir, cel_files_pattern)
  } else {
    in_files
  }
  library(simpleaffy)
  Cel=ReadAffy(filenames = in_files, celfile.path = in_dir)
  E = mas5(Cel, sc=1000)
  outfile <- paste(out_prefix, "mas5.csv", sep = ".")
  write.exprs(E, file=outfile)
  as.integer(ncol(E))
}

run_submean2 <- function(io_prefix){
  infile <- paste(io_prefix, "mas5.csv", sep = ".")
  outfile <- paste(io_prefix, "log2.csv", sep = ".")
  Tab=read.table(infile)
  Tab=log2(Tab)
  Tab=Tab-apply(Tab, 1, mean)
  Tab=round(Tab, digits=5)
  colnames(Tab)=gsub(".CEL", "", colnames(Tab))
  write.table(file=outfile, Tab, row.names=TRUE,
              col.names=TRUE, sep="\t", quote=FALSE)
  ncol(Tab)
}

run_mas_submean <- function(db, id){
  dir_name <- paste(data.final.dir, db, id, sep = "/")
  io_prefix <- paste(dir_name, id, sep = "/")
  mas_result <- run_mas5(dir_name, io_prefix)
  submean_result <- run_submean2(io_prefix)
  c( mas_result = mas_result,
    submean_result = submean_result)
}


run_final_mas5_submean <- function(final_counts_df){
  ddply( final_counts_df, .(db, id), function(x){
    db <- x[1,1]
    id <- x[1,2]
    rv <- try(run_mas_submean(db, id),
              silent = T)
    if(class(rv) == "try-error"){
      rv <- c( mas_result = -1, submean_result = -1)
    }
    write.table(data.frame(t(c(db = db, id = id, rv))) ,
                "mas_submean_progress.txt",
                col.names = F, row.names = F, append = T, sep = "|")
    rv
  })
}

run_mas_submean_bg <- function(){
  final_count <- count_final_dir(c("NASC","ArrayExpress","TAIR","GEO"))
  fcr <- count_cel_files(final_count)
  bgd <- run_final_mas5_submean(fcr)
  write.table(bgd, "mas_submea_result.txt",
              sep = "|", quote = F, row.names = F)
}

csv_combine <- function(in_files, out_file){
  tab=read.table(in_files[1])
  for (i in in_files[-1]) {
    print(i)
    x=read.table(i)
    tab=cbind(tab,x)
  }
  write.table(tab, file=out_file, sep="\t", quote=FALSE)
  nrow(tab)
}

csv_combine2 <- function(in_files, out_file, rename_columns = F,
                         dest_prefix = "SR_"){
  process_fcn <- function(cfn, append = T,
                          col.names = F) {
    print(cfn)
    rtable <- read.table(cfn, header = T, as.is = T)
    mdim <- dim(rtable)
    stopifnot(dim(rtable)[1] == mdim[1])
    access_name <- sub(dest_prefix, "", unlist(strsplit(basename(cfn), "\\."))[1]);
    ncols <- length(colnames(rtable))
    pcolumns <- colnames(rtable)
    if(rename_columns){
      colnames(rtable) <- paste(rep(access_name, ncols), 1:ncols, sep = "_")
    }
    write.table(t(rtable),out_file, row.names = T, col.names = col.names,
                quote = F, append = append)
    data.frame(Access = rep(access_name, ncols), old_cols = pcolumns,
               new_cols = colnames(rtable), rows = rep(mdim[1], ncols),
               cols = rep(mdim[2], ncols))
  }
  ret1_df <- process_fcn(in_files[1], append = F, col.names = T)
  ret_df <- ldply(in_files[-1], process_fcn)
  if(length(in_files) > 1){
    ftab <- read.table(out_file, header = T, as.is = T, check.names = F)
    stopifnot(length(unique(rownames(ftab))) == length(rownames(ftab)))
    write.table(t(ftab),out_file, row.names = T,
                col.names = T, quote = F)
  }
  rbind(ret1_df, ret_df)
}

quantile_normalize <- function(infile, outfile) {
  library(affy)
  library(limma) # get arguments

  D0 = read.table(infile, head=TRUE, row.names=1) # get data layout
  RNAMES=rownames(D0)
  CNAMES=colnames(D0)
  N = ncol(D0)
  M = nrow(D0)
  write.table(file=outfile, D0, row.names=FALSE,
              col.names=FALSE, sep="\t", quote=FALSE)

  D0 = matrix(scan(outfile, n = N * M), # read and normalize
    M, N, byrow = TRUE)
  D0 = normalizeBetweenArrays(D0, method="quantile")
  D0=round(D0, digits=5)

  rownames(D0)=RNAMES # write
  colnames(D0)=CNAMES
  write.table(file=outfile, D0, row.names=TRUE,
              col.names=TRUE, sep="\t", quote=FALSE)
  nrow(D0)
}

iqr_filter <- function(infile, outfile, iqr){
  M=read.table(infile, head=T, row.names=1)
  Iqrs=apply(M, 1, IQR)
  Midx=as.vector(which(Iqrs > iqr, arr.ind=T))

  write.table(file=outfile, M[Midx,], sep="\t", quote=F)
  c(iqr_in = nrow(M), iqr_out = nrow(M[Midx,]))
}

iqr_filter_dry <- function(infile, iqr){
  M=read.table(infile, head=T, row.names=1)
  Iqrs=apply(M, 1, IQR)
  Midx=as.vector(which(Iqrs > iqr, arr.ind=T))
  print(c(iqr_in = nrow(M), iqr_out = sum(Iqrs > iqr)))
  Midx
}

get_submean_files_fcr <- function(fcr){
  submean_files <- dlply( fcr, .(db, id), function(x){
    db <- x[1,1]
    id <- x[1,2]
    dir_name <- paste(data.final.dir, db, id, sep = "/")
    io_prefix <- paste(dir_name, id, sep = "/")
    paste(io_prefix, "log2.csv", sep = ".")
  })
  unlist(submean_files)
}

get_submean_files <- function(){
  final_count <- count_final_dir(c("NASC","ArrayExpress","TAIR","GEO"))
  fcr <- count_cel_files(final_count)
  get_submean_files_fcr(fcr)
}

normalize_final <- function() {
  io_prefix <- paste(data.final.dir, "final_", sep = "/")
  combine_file <- paste(io_prefix,"combine.csv", sep = "")
  quantile_file <- paste(io_prefix, "quantile.csv", sep = "")
  iqr_file <- paste(io_prefix, "iqr.csv", sep = "")
  submean_files <- get_submean_files()
  cr <- csv_combine(submean_files, combine_file)
  qr <- quantile_normalize(combine_file, quantile_file)
  ir <- 0
  #ir <- iqr_filter(quantile_file, iqr_file, 0.1)
  c(combine_result = cr, quantile_result = qr, ir)
}

cels_mas5_submean <- function(cel_files_data, root_dir,
                              out_base_dir,
                              dest_prefix) {
  ddply(cel_files_data, .(Database, Access), function(sub_df) {
    data_source <- sub_df[1, "Database"]
    access_name <- sub_df[1, "Access"]
    in_files <- as.vector(sub_df[, "CEL"])
    in_dir <- paste(root_dir, sub_df[1, "Database"], sub_df[1, "Access"], "", sep = "/")
    io_prefix <- paste(out_base_dir, paste(dest_prefix, access_name, sep = "" ),
                       sep = "/")
    print(in_dir)
    #print(io_prefix)
    #print(in_files)
    rv1 <- try(run_mas5(in_dir, io_prefix, in_files = in_files),
                      silent = T)
    rv2 <- if(!(class(rv1) == "try-error")){
      try(run_submean2(io_prefix), silent = T)
    } else {
      -1
    }
    if(class(rv1) == "try-error" | class(rv2) == "try-error"){
      crv <- c(db = data_source, idx = access_name, ncels = nrow(sub_df),
              mas_result = -1, submean_result = -1)
      write.table(data.frame(t(crv)) ,"mas5_run_progress.txt",
                  col.names = F, row.names = F, append = T, sep = "|", quote = F)
      crv
    } else {
      crv <- c(db = data_source, idx = access_name, ncels = nrow(sub_df),
              mas_result = rv1, submean_result = rv2)
      write.table(data.frame(t(crv)) ,"mas5_run_progress.txt",
                  col.names = F, row.names = F, append = T, sep = "|", quote = F)
      crv
    }
  })
}


csv2exp <- function(csv_file, map_file, match = F, delim = "", sep = "\t"){
  map_table <- read.table(map_file, header = F, as.is = T)
  colnames(map_file) <- c("probe", "id")
}


qn_iqr_rows <- function(ematrix, sel_cols, iqr) {
  library(affy)
  library(limma) # get arguments

  M = normalizeBetweenArrays(ematrix[,sel_cols], method="quantile")
  Iqrs=apply(M, 1, IQR)
  Midx=as.vector(which(Iqrs > iqr, arr.ind=T))
  print(c(iqr_in = nrow(M), iqr_out = sum(Iqrs > iqr)))
  Midx
}

qn_normalized_profile <- function(ematrix, sel_cols, sel_rows){
  library(affy)
  library(limma) # get arguments

  M = normalizeBetweenArrays(ematrix[,sel_cols], method="quantile")
  M[sel_rows,]
}

qn_iqr_vals <- function(ematrix, sel_cols, sel_rows ) {
  library(affy)
  library(limma) # get arguments

  M = normalizeBetweenArrays(ematrix[,sel_cols], method="quantile")
  Iqrs=apply(M, 1, IQR)
  Iqrs[sel_rows]
}

qn_iqr_dry <- function(ematrix, sel_cols, iqr) {
  library(affy)
  library(limma) # get arguments

  M = normalizeBetweenArrays(ematrix[,sel_cols], method="quantile")
  Iqrs=apply(M, 1, IQR)
  Midx=as.vector(which(Iqrs > iqr, arr.ind=T))
  print(c(iqr_in = nrow(M), iqr_out = sum(Iqrs > iqr)))
  Midx
}

qn_iqr_auto_df <- function(iqr_vals, binwidth = 0.05){
  istart <- round(min(iqr_vals), digits = 1)
  iend <- round(max(iqr_vals), digits = 1) + binwidth
  iqr_breaks <- seq(from = istart, to=iend, by = binwidth)
  nbreaks <- length(iqr_breaks)
  iqr_labels <- iqr_breaks[2:nbreaks]
  iqr_tb <- table(cut(iqr_vals,
                      breaks=iqr_breaks,
                      labels=iqr_labels))
   data.frame(iqr = iqr_labels, freq = as.vector(iqr_tb))
}

qn_iqr_auto <- function(iqr_vals, binwidth = 0.05){
  istart <- round(min(iqr_vals), digits = 1)
  iend <- round(max(iqr_vals), digits = 1) + binwidth
  iqr_breaks <- seq(from = istart, to=iend, by = binwidth)
  nbreaks <- length(iqr_breaks)
  iqr_labels <- iqr_breaks[2:nbreaks]
  iqr_tb <- table(cut(iqr_vals,
                      breaks=iqr_breaks,
                      labels=iqr_labels))
  iqr_df <- data.frame(iqr = iqr_labels, freq = as.vector(iqr_tb))
  iqr_df$iqr[iqr_df$freq == max(iqr_df$freq)]
}

columns.iqr <- function(ematrix, sel.cols){
  library(affy)
  library(limma) # get arguments

  M = normalizeBetweenArrays(ematrix[,sel.cols], method="quantile")
  apply(M, 1, IQR)
}

columns.iqr.df <- function(ematrix, sel.cols){
    col.iqrs <- columns.iqr(ematrix, sel.cols)
    qn_iqr_auto_df(col.iqrs)
}


qn_iqr_dry_auto <- function(ematrix, sel_cols) {
  library(affy)
  library(limma) # get arguments

  M = normalizeBetweenArrays(ematrix[,sel_cols], method="quantile")
  Iqrs=apply(M, 1, IQR)
  iqr = qn_iqr_auto(Iqrs)
  if(length(iqr) > 1){
    iqr <- iqr[length(iqr)]
  }
  Midx=as.vector(which(Iqrs > iqr, arr.ind=T))
  print(c(iqr_v = iqr, iqr_in = nrow(M), iqr_out = sum(Iqrs > iqr)))
  list(idx = Midx, iqr = iqr)
}

qn_iqr <- function(ematrix, sel_cols, iqr, fname) {
  Midx = qn_iqr_rows(ematrix, sel_cols, iqr)
  write.table(file=fname, ematrix[Midx, sel_cols],
              row.names=TRUE, col.names=TRUE,
              sep="\t", quote=FALSE)
}
