source("io_env.R")
source("cel_data.R")
source("preprocess.R")
source("normalize_exp.R")
source("combine_exp.R")

## Expected input format
## Make sure all the variables in io_env.R are properly set

## 0. Pre-process : Compute NUSE and RLE
preprocess.main <- function(){
  acc.df <- load.access.ids()
  ## database of CEL files
  cel.names.df <- get.cel.filenames(acc.df, cel.names.file)
  write.pipe.df(cel.names.df, cel.names.file)
  ## Compute NUSE and RLE
  preprocess_cels(acc.df, data.root.dir)
  ## Retrieve the CELs failing RLE and NUSE
  pp.fail.df <- get.summary.data(acc.df, data.root.dir)
  ## Write the failures
  write.pipe.df(pp.fail.df, preprocess.fail.file)
}

## 1. Reject bad files (RLE/NUSE fails and Manual Inspection)
reject.main <- function (){
  ## load the complete CEL list
  cel.names.df <- read.pipe.df(cel.names.complete.file)
  ## load the rejected from RLE/NUSE inspection
  pp.fail.df <- read.pipe.df(preprocess.fail.file)
  ## load the results from manual inspection
  man.fail.df <- read.pipe.df(man.inspect.fail.file)
  man.except.df <- read.pipe.df(man.inspect.except.file)

  ## merge the data
  cel.names.ndf <- setdiff(cel.names.df,
                           union(setdiff(pp.fail.df, man.except.df),
                                 man.fail.df))

  ## write the cel metadata
  write.pipe.df(cel.names.ndf, cel.names.file)
}

## 2. Normalize within each Experiment and copy the final list
normalize.main <- function(){
  acc.df <- load.access.ids()

  ## Final normalization
  cel.names.df <- read.pipe.df(cel.names.file)
  cel.normalize.results <- cels_mas5_submean(
    cel.names.df, data.root.dir, final.norm.dir, "")
  if(any(cel.normalize.results$mas_result == -1) ||
     any(cel.normalize.results$submean_result == -1)){
    write.pipe.df(cel.normalize.results, cel.errors.file)
    print("Errors in Normalization. Check cel.errors.file. \n")
    FALSE
  } else {

    ## copy zola's files to normalize directory
    zola.data.csv <- dir(zola.data.dir, "*.log2.csv")
    zola.copied.out <- ldply(zola.data.csv, function(fx){
      copy.norm.file(fx, zola.data.dir, final.norm.dir, "")
    })
    TRUE
  }
}

## 3. Combine normalized CEL values to the final csv
combine.main <- function() {
  norm.root <- final.norm.dir
  norm.csv.files <- dir(norm.root, "*.log2.csv", full.names = T)
  csv_combine2(norm.csv.files, combo.csv.file,
               rename_columns = T, dest_prefix = "")
}

## 4. Select rows based on classification
classify.main <- function(){
  ## load complete data
  complete.data <- read.table(combo.csv.file, row.names = 1, head = T)
  cdata.matrix <- as.matrix(complete.data)

  ## column information
  colmap.df = read.comma.df(columns.class.map.file)
  for(cx in class.names){
    ## column identifiers
    cxmap.cids <- as.integer(colmap.df[colmap.df[,c(cx)] == "True",
                                       "Column"])

    ## automatic IQR for classification files
    cx.iqr <- qn_iqr_dry_auto(cdata.matrix, cxmap.cids)

    ## Quantile normalize
    Mx <- normalizeBetweenArrays(cdata.matrix[cx.iqr$idx, cxmap.cids],
                                 method="quantile")

    ## Write quantile normalized files
    cx.fname <- paste(tp.norm.dir, cx, csv.qn.file.suffix, sep="")
    write.table(file=cx.fname, Mx,
                row.names=TRUE, col.names=TRUE,
                sep="\t", quote=FALSE)
  }
}

## 5. Update the probe maps
probes.main <- function(){

  csv.qn.names = paste(class.names, csv.qn.file.suffix, sep = "")
  csv.qn.file.paths = paste(tp.norm.dir, csv.qn.names, sep = "")
  tmp.exp.names = paste(class.names, tmp.exp.file.suffix, sep = "")
  tmp.exp.file.paths = paste(tp.norm.dir, tmp.exp.names, sep = "")
  exp.names = paste(class.names, exp.file.suffix, sep = "")
  exp.file.paths = paste(tp.norm.dir, exp.names, sep = "")
  exp.log.names = paste(class.names, "-exp-clean.log", sep = "")
  exp.log.file.paths = paste(tp.norm.dir, exp.log.names, sep = "")
  ## convert to exp file
  for(i in (1:length(class.names))){
    ## ./csv2exp --map probe.id.file x-dataset-qn.csv
    csvexp.run = paste(csv2exp.cmd, "--map", probe.id.file,
                       csv.qn.file.paths[i])
    system(csvexp.run)

    ## grep -v AFFX $out.exp > $tmp
    grep.run = paste("grep -v AFFX", exp.file.paths[i], ">",
                     tmp.exp.file.paths[i])
    system(grep.run)

    ## ./exp-clean-ht $tmp $out.exp > $out.log
    exp.clean.run = paste(exp.clean.cmd, tmp.exp.file.paths[i],
                          exp.file.paths[i], ">",
                          exp.log.file.paths[i])
    system(exp.clean.run)

    ##  rm $tmp
    rmtmp.run = paste("rm", tmp.exp.file.paths[i])
    system(rmtmp.run)
  }

}

## 0. Pre-process : Compute NUSE and RLE
#preprocess.main()
## 1. Reject bad files (RLE/NUSE fails and Manual Inspection)
#reject.main()
## 2. Normalize within each Experiment and copy the final list
#normalize.main()
## 3. Combine normalized CEL values to the final csv
#combine.main()
## 4. Select rows based on classification
#classify.main()
## 5. Update the probe maps
#probes.main()
