library("plyr")
source("io_env.R")

get_cel_files <- function(dir_name){
  cel_files <- dir(dir_name, cel.files.pattern)
  unlist(llply(cel_files, function(x){
    paste(dir_name, "/", x, sep = "")
  }))
}

run_affyqc <- function(in_dir, io_prefix,
                       in_files = character(0) ){
  in_files <- if(length(in_files) == 0) {
    dir(in_dir, cel.files.pattern)
  } else {
    in_files
  }
  library(simpleaffy)
  Cel=ReadAffy(filenames = in_files,
    celfile.path = in_dir)
  Qc=qc(Cel)

  M=Qc@average.background
  M=cbind(M, Qc@scale.factors)

  ac3ac5=Qc@qc.probes[,1]/Qc@qc.probes[,2]
  g3g5=Qc@qc.probes[,4]/Qc@qc.probes[,5]

  M=cbind(M, ac3ac5)
  M=cbind(M, g3g5)
  M=round(M, digits=5)
  M=cbind(M, Qc@bioBCalls)

  outqc=paste(io_prefix, "qc", sep=".")
  write.table(M, outqc, col.names=FALSE, sep="\t", quote=FALSE)
  nrow(M)
}

plm_stat <- function(in_dir, io_prefix,
                     in_files = character(0) ){
  library(affyPLM)
  in_files <- if(length(in_files) == 0) {
    dir(in_dir, cel.files.pattern)
  } else {
    in_files
  }
  Cel=ReadAffy(filenames = in_files,
    celfile.path = in_dir)
  Pset=fitPLM(Cel)

  # *** RLE ***
  Rle=RLE(Pset, type="stats")

  M=t(Rle)
  M=round(M, digits=5)

  m=nrow(M)
  M=cbind(M, rep(1, m))

  Mean=mean(M[,1])
  Sd=sd(M[,1])

  for (i in 1:m) {
    if (abs(M[i,1]-Mean) > (1.75 * Sd)) M[i,3]=0
    if (abs(M[i,2]) > 0.5) M[i,3]=0
  }

  outRLE=paste(io_prefix, "rle", sep=".")
  write.table(M, outRLE, col.names=FALSE, sep="\t", quote=FALSE)

                                        # *** NUSE ***
  Nuse=NUSE(Pset, type="stat")
  M=t(Nuse)
  M=round(M, digits=5)

  m=nrow(M)
  M=cbind(M, rep(1,m))

  Mean=mean(M[,1])
  Sd=sd(M[,1])

  for (i in 1:m) {
    if (abs(M[i,1]-Mean) > (1.75 * Sd)) M[i,3]=0
    if (abs(M[i,2]) > 0.5) M[i,3]=0
  }

  outNUSE=paste(io_prefix, "nuse", sep=".")
  write.table(M, outNUSE, col.names=FALSE, sep="\t", quote=FALSE)
  nrow(M)
}

rma_expression_set <- function(in_dir, io_prefix,
                               in_files = character(0)){
  cwd <- getwd()
  library(affy)
  in_files <- if(length(in_files) == 0) {
    dir(in_dir, cel.files.pattern)
  } else {
    in_files
  }
  Eset=justRMA(filenames = in_files, celfile.path = in_dir)
  D=exprs(Eset)
  D=2^D
  D=round(D, digits=5)
  outfile=paste(io_prefix, "csv", sep=".")
  write.table(D, file=outfile, sep="\t", quote=F)
  nrow(D)
}

run_dres <-  function(io_prefix){
  in_file1 <- paste(io_prefix, "csv", sep = ".")
  in_file2 <- paste(io_prefix, "exp", sep = ".")
  out_file <- paste(io_prefix, "dres", sep = ".")

  csv2exp_syscmd <- paste(csv2exp.cmd, in_file1, sep = " ")
  csv2exp_ret <- system(csv2exp_syscmd)

  residuals_syscmd <- paste(residuals.cmd, in_file2, out_file, sep = " ")
  residuals_ret <-  system(residuals_syscmd)
  c(residuals = residuals_ret, csv_exp = csv2exp_ret)
}

run_dres2 <- function(io_prefix) {
  csv_file <- paste(io_prefix, "csv", sep = ".")
  out_file <- paste(io_prefix, "dres", sep = ".")
  expr_data <- as.matrix.data.frame(read.table(csv_file, sep = "\t", header = T))

  D <- aaply(expr_data, 1, function(x) {
    r <- x - mean(x);
    n <- length(x) ;
    sse <- sum(r * r) * (1 - (1.0/n));
    r * sqrt(n/(sse - (r * r)))})
  D <- round(D, digits=6)
  write.table(D, file = out_file, sep = "\t", quote = F)
  c( residuals = 0, csv_exp = 0)
}

run_kstest <- function(io_prefix){
  in_file <- paste(io_prefix, "dres", sep = ".")
  D = read.table(in_file, head=TRUE, row.names=1)

  m = ncol(D)
  df = m - 2

  M = matrix(nrow=m, ncol=2)

  for (i in 1:m) {
    res = ks.test(D[,i], "pt", df)
    M[i,1] = res$stat
    M[i,2] = 1;
    if (M[i,1] > 0.15) M[i,2] = 0;
  }

  M[,1] = round(M[,1], digits=5)
  outfile <- paste(io_prefix, "ks", sep = ".")
  write.table(M, outfile, col.names=FALSE, sep="\t", quote=FALSE)
  nrow(M)
}


do_summary <- function(io_prefix){

  Rle=read.table(paste(io_prefix, "rle", sep="."), head=F)
  Nuse=read.table(paste(io_prefix, "nuse", sep="."), head=F)
  Qc=read.table(paste(io_prefix, "qc", sep="."), head=F)
  Ks=read.table(paste(io_prefix, "ks", sep="."), head=F)
  List=list() # Here analysis begins
  m=nrow(Rle) # RLE
  Mean=mean(Rle[,2])
  Sd=sd(Rle[,2])
  IQR_Mean=mean(Rle[,3])
  IQR_Sd=sd(Rle[,3])

  for (i in 1:m) {
    Name=as.character(Rle[i,1])
    # if (abs(Rle[i,2]-Mean) > (1.75 * Sd)) List[Name]=Name
    # if (abs(Rle[i,3]-IQR_Mean) > (1.75 * IQR_Sd)) List[Name]=Name
    if (abs(Rle[i,2]) > 0.075) List[Name]=Name
    if (abs(Rle[i,3]) > 0.75) List[Name]=Name
  }

  Mean=mean(Nuse[,2]) # NUSE
  Sd=sd(Nuse[,2])
  IQR_Mean=mean(Nuse[,3])
  IQR_Sd=sd(Nuse[,3])

  for (i in 1:m) {
    Name=as.character(Rle[i,1])
    # if (abs(Nuse[i,2]-Mean) > (1.75 * Sd)) List[Name]=Name
    # if (abs(Nuse[i,3]-IQR_Mean) > (1.75 * IQR_Sd)) List[Name]=Name
    if (abs(Nuse[i,2]-1) > 0.075) List[Name]=Name
    if (abs(Nuse[i,3]) > 0.75) List[Name]=Name
  }

  # QC BioB
  for (i in 1:m) {
    Name=as.character(Rle[i,1])
    if (as.character(Qc[i,6]) == "A") List[Name]=Name
  }

  Qc[,3]=abs(log2(Qc[,3])) # QC scale
  for (i in 1:m) {
    Name=as.character(Rle[i,1])
    if (Qc[i,3] > 3) List[Name]=Name
  }

  Lv = as.vector(List)
  write.table(Lv, paste(io_prefix, "sum", sep="."),
              col.names=F, row.names=F, quote=F, sep="\n")
  Lv
}


do_report <- function(Lv,in_dir,io_prefix,
                      in_files = character(0),
                      force = FALSE){
  in_files <- if(length(in_files) == 0) {
    dir(in_dir, cel.files.pattern)
  } else {
    in_files
  }
  if(length(Lv) > 0 | force == TRUE) {
    library(affyPLM)
    library(RColorBrewer)
    Cols=brewer.pal(12,"Set3")
    E=ReadAffy(filenames = in_files, celfile.path = in_dir)
    Pset=fitPLM(E)
    out_file <- paste(io_prefix, "_RLE_NUSE.pdf", sep = "")
    pdf(out_file, width=20)
    RLE(Pset, main="Relative Log Expression", col=Cols)
    NUSE(Pset, main="Normalized Unscaled Standard Errors", col=Cols)
    dev.off()

  }
  if(length(Lv) > 0) {
    0
  } else {
    1
  }
}

create_links <- function(Lv,in_dir, data_source, db_id){
  #TODO:
  cwd <- getwd()
  final_dir_dsid <- paste(data.final.dir, data_source, db_id, sep = "/")
  system(paste("mkdir -p", final_dir_dsid, sep = " "))
  cel_files <- dir(in_dir,"*.CEL$")
  ncel <- length(cel_files)
  setwd(final_dir_dsid)
  count <- 0
  lfc <- 0
  for(i in 1:ncel){
    if(!(cel_files[i] %in% Lv )){
      fname <- cel_files[i]
      fname <- gsub("\\(","\\\\(",fname)
      fname <- gsub("\\)","\\\\)",fname)
      cel_file_path <- paste(in_dir, fname, sep = "/")
      ret <- system(paste("ln -s", cel_file_path, fname))
      count <- count + 1
      if(ret != 0){
        lfc <- lfc + 1
      }
    }
  }
  setwd(cwd)
  c(link_count = count, link_fails = lfc)
}

preprocess_qc <- function(in_dir, db_id, data_source,
                          io_prefix = character(0),
                          in_files = character(0),
                          force = FALSE,
                          run_type = "dry") {
  io_prefix <- if(length(io_prefix) == 0) {
    paste(in_dir, db_id, sep = "/")
  } else {
    io_prefix
  }
  # Step 1
  r1 = run_affyqc(in_dir, io_prefix, in_files = in_files)
  # Step 2
  r2 = plm_stat(in_dir, io_prefix, in_files = in_files)
  # Step 3
  r3 = rma_expression_set(in_dir, io_prefix, in_files = in_files)
  r4 = run_dres2(io_prefix)
  r5 = run_kstest(io_prefix)
  # step 4
  Lv = do_summary(io_prefix)
  r6 = length(Lv)
  # step 5
  r7 = do_report(Lv, in_dir, io_prefix,
    in_files = in_files, force = force)
  # step 6
  if(run_type == "dry"){
    r8 = c(link_count = 0, link_fails = 0)
  } else {
    r8 = create_links(Lv, in_dir, data_source, db_id)
  }
  c(aff_qc = r1, plm_stat = r2, rma = r3,  r4,
    kstest = r5, summary = r6, report = r7, r8)
}

unzip_nasc <- function(nasc_base_dir,nasc_ids){
  cwd <- getwd()
  result <- ldply(nasc_ids,function(x){
    nasc_dir <- paste(nasc_base_dir,x, sep="/")
    setwd(nasc_dir)
    r <- 0
    #r <- system(paste("gunzip *.CEL.gz"))
    cfs <- dir(".","*.CEL$")
    cfs2 <- dir(".", "*.cel$")
    c(x,r,length(cfs),length(cfs2))
  })
  setwd(cwd)
  result
}

unzip_aexp <- function(aexp_base_dir,aexp_ids){
  cwd <- getwd()
  result <- ldply(aexp_ids,function(x){
    aexp_dir <- paste(aexp_base_dir,x, sep="/")
    setwd(aexp_dir)
    r <- 0
    raw_zips <- dir(".", "*.raw.*.zip")
    for(rf in raw_zips) {
      sf <- 0
      #sf <- system(paste("unzip -o ", rf))
      if(sf != 0 & r != 0) {
        r <- sf
      }
    }
    cfs <- dir(".","*.CEL$")
    cfs2 <- dir(".", "*.cel$")
    c(x,r,length(cfs),length(cfs2))
  })
  setwd(cwd)
  result
}


unzip_tair <- function(tair_base_dir,tair_ids){
  cwd <- getwd()
  result <- ldply(tair_ids,function(x){
    tair_dir <- paste(tair_base_dir,x, sep="/")
    setwd(tair_dir)
    r <- 0
    raw_zips <- dir(".", "*.zip")
    for(rf in raw_zips) {
      sf <- 0
      sf <- system(paste("unzip -o ", rf))
      if(sf != 0 & r != 0) {
        r <- sf
      }
    }
    cfs <- dir(".","*.CEL$")
    cfs2 <- dir(".", "*.cel$")
    c(x,r,length(cfs),length(cfs2))
  })
  setwd(cwd)
  result
}

rename_tair <- function(tair_base_dir,tair_ids){
  cwd <- getwd()
  result <- ldply(tair_ids,function(x){
    tair_dir <- paste(tair_base_dir,x, sep="/")
    setwd(tair_dir)
    r <- 0
    raw_cels <- dir(".", "*.cel$")
    for(rf in raw_cels) {
      sf <- 0
      nrf <- paste(basename(rf), "CEL", sep = ".")
      sf <- system(paste("mv ", rf, nrf))
      if(sf != 0 & r != 0) {
        r <- sf
      }
    }
    cfs <- dir(".","*.CEL$")
    cfs2 <- dir(".", "*.cel$")
    c(x,r,length(cfs),length(cfs2))
  })
  setwd(cwd)
  result
}

unzip_geo <- function(geo_base_dir,geo_ids) {
  cwd <- getwd()
  result <- ldply(geo_ids,function(x){
    geo_dir <- paste(geo_base_dir,x, sep="/")
    setwd(geo_dir)
    r <- 0
    raw_zips <- dir(".", "*.tar")
    for(rf in raw_zips) {
      sf <- 0
#      sf <- system(paste("tar -xvf ", rf))
      if(sf != 0 & r != 0) {
        r <- sf
      }
    }
    raw_zips <- dir(".", "*.gz")
    for(rf in raw_zips) {
      sf <- 0
#      sf <- system(paste("gunzip ", rf))
      if(sf != 0 & r != 0) {
        r <- sf
      }
    }
    cfs <- dir(".","*.CEL$")
    cfs2 <- dir(".", "*.cel$")
    c(x,r,length(cfs),length(cfs2))
  })
  setwd(cwd)
  result
}
del_geo <- function(geo_base_dir, geo_ids){
  cwd <- getwd()
  result <- ldply(geo_ids,function(x){
    geo_dir <- paste(geo_base_dir,x, sep="/")
    setwd(geo_dir)
    r <- 0
    raw_zips <- dir(".", "*.gz")
    for(rf in raw_zips) {
      sf <- 0
      sf <- system(paste("rm -f ", rf))
      if(sf != 0 & r != 0) {
        r <- sf
      }
    }
    cfs <- dir(".","*.CEL$")
    cfs2 <- dir(".", "*.cel$")
    c(x,r,length(cfs),length(cfs2))
  })
  setwd(cwd)
  result
}
nasc_preprocess <- function(nasc_base_dir,nasc_ids,write_data = F){
  result <- ldply(nasc_ids,function(x){
    nasc_exp_dir <- paste(nasc_base_dir,x,sep = "/")
    rv <- try(preprocess_qc(nasc_exp_dir, x, "NASC"),
              silent = T)
    if(class(rv) == "try-error"){
      c(id = x, aff_qc = -1, plm_stat = -1, rma = -1,
        residuals = -1, csv_exp = -1,
        kstest = -1, summary = -1, report = -1,
        link_count = -1, link_fails = -1)
    } else {
      c(id = x, rv)
    }
  })
  if(write_data) {
    write.table( result, "nasc_preprocess_report.txt", sep="|", quote=F,
                row.names = F)
  }
  result

}

aexp_preprocess <- function(aexp_base_dir,aexp_ids,write_data = F,
                            write_fname = "aexp_preprocess_report.txt"){
  frv <- c( aff_qc = -1, plm_stat = -1, rma = -1,
           residuals = -1, csv_exp = -1,
           kstest = -1, summary = -1, report = -1,
           link_count = -1, link_fails = -1)
  ncols <- length(frv) + 1
  result <- ldply(aexp_ids,function(x){
    aexp_exp_dir <- paste(aexp_base_dir,x,sep = "/")
    rv <- try(preprocess_qc(aexp_exp_dir, x, "ArrayExpress"),
              silent = T)
    if(class(rv) == "try-error"){
      write.table(data.frame(t(c(idx = x, frv))) ,"run_progress.txt",
                  col.names = F, row.names = F, append = T,
                  sep = "|", quote = F)
      c(idx = x, frv)
    } else {
      write.table(data.frame(t(c(idx = x, rv))) ,"run_progress.txt",
                  col.names = F, row.names = F, append = T,
                  sep = "|", quote = F)
      c(id = x, rv)
    }
  })
  if(write_data) {
    write.table(result, write_fname, sep="|", quote=F,
                row.names = F)
  }
  result

}


tair_preprocess <- function(tair_base_dir,tair_ids,write_data = F,
                            write_fname = "tair_preprocess_report.txt"){
  frv <- c( aff_qc = -1, plm_stat = -1, rma = -1,
           residuals = -1, csv_exp = -1,
           kstest = -1, summary = -1, report = -1,
           link_count = -1, link_fails = -1)
  ncols <- length(frv) + 1
  result <- ldply(tair_ids,function(x){
    tair_exp_dir <- paste(tair_base_dir,x,sep = "/")
    rv <- try(preprocess_qc(tair_exp_dir, x, "TAIR"),
              silent = T)
    if(class(rv) == "try-error"){
      write.table(data.frame(t(c(idx = x, frv))) ,"run_progress.txt",
                  col.names = F, row.names = F, append = T,
                  sep = "|", quote = F)
      c(idx = x, frv)
    } else {
      write.table(data.frame(t(c(idx = x, rv))) ,"run_progress.txt",
                  col.names = F, row.names = F, append = T,
                  sep = "|", quote = F)
      c(id = x, rv)
    }
  })
  if(write_data) {
    write.table(result, write_fname, sep="|", quote=F,
                row.names = F)
  }
  result

}

geo_preprocess <- function(geo_base_dir,geo_ids,write_data = F,
                            write_fname = "geo_preprocess_report.txt"){
  frv <- c( aff_qc = -1, plm_stat = -1, rma = -1,
           residuals = -1, csv_exp = -1,
           kstest = -1, summary = -1, report = -1,
           link_count = -1, link_fails = -1)
  ncols <- length(frv) + 1
  result <- ldply(geo_ids,function(x){
    geo_exp_dir <- paste(geo_base_dir,x,sep = "/")
    rv <- try(preprocess_qc(geo_exp_dir, x, "GEO"),
              silent = T)
    if(class(rv) == "try-error"){
      write.table(data.frame(t(c(idx = x, frv))) ,"run_progress.txt",
                  col.names = F, row.names = F, append = T,
                  sep = "|", quote = F)
      c(idx = x, frv)
    } else {
      write.table(data.frame(t(c(idx = x, rv))) ,"run_progress.txt",
                  col.names = F, row.names = F, append = T,
                  sep = "|", quote = F)
      c(id = x, rv)
    }
  })
  if(write_data) {
    write.table(result, write_fname, sep="|", quote=F,
                row.names = F)
  }
  result

}


list_dir <- function(dir_name){
  cel_files <- dir( dir_name, cel.files.pattern )
  ldply(cel_files, function(z) {
    c(fname = z)
  })
}

list_dir_cels_df <- function(ddf, root_dir = data.final.dir){
  ddply(ddf, .( Database, Access ), function(x) {
    db_name <- x[1, "Database"]
    access_name <- x[1, "Access"]
    access_dir <- paste(root_dir, db_name, access_name, sep = "/")
    list_dir(access_dir)
  })
}

list_dir_cels <- function( db_ids, root_dir = data.final.dir){
  ldply( db_ids, function(x) {
    db_root_dir <- paste(root_dir, x, sep = "/")
    dbid_dirs <- dir(db_root_dir)
    ldply(dbid_dirs, function(y) {
      dbid_cel_dir <- paste( db_root_dir, y, sep = "/")
      cel_files <- dir( dbid_cel_dir, cel.files.pattern )
      ldply(cel_files, function(z) {
        c(db = x, id = y, fname = z)
      })
    })
  })
}

count_cel_files <- function(final_data_df) {
  ddply( final_data_df, .( db, id ), nrow)
}

preprocess_cels <- function(cel_files_data, root_dir,
                            dest_prefix) {
  frv <- c( aff_qc = -1, plm_stat = -1, rma = -1,
           residuals = -1, csv_exp = -1,
           kstest = -1, summary = -1, report = -1,
           link_count = -1, link_fails = -1)
  ddply(cel_files_data, .(Database, Access), function(sub_df){
    data_source <- sub_df[1, "Database"]
    access_name <- sub_df[1, "Access"]
    in_files <- as.vector(sub_df[, "CEL"])
    in_dir <- paste(root_dir, sub_df[1, "Database"], sub_df[1, "Access"], "", sep = "/")
    io_prefix <- paste(in_dir, dest_prefix, access_name, sep = "")
    print(io_prefix)
    rv <- try(preprocess_qc(in_dir, access_name, data_source,
                            in_files = in_files, io_prefix = io_prefix),
              silent = T)
    if(class(rv) == "try-error"){
      write.table(data.frame(t(c(db = data_source, idx = access_name, frv))),
                  "run_progress.txt",
                  col.names = F, row.names = F, append = T,sep = "|", quote = F)
      c(db = data_source, idx = access_name, frv)
    } else {
      write.table(data.frame(t(c(db = data_source, idx = access_name, rv))),
                  "run_progress.txt",
                  col.names = F, row.names = F, append = T, sep = "|", quote = F)
      c(db = data_source, idx = access_name, rv)
    }
  })
}
