library(plyr)
library(ggplot2)
source("normalize_exp.R")

iqr.deriv.df <- function(gene.summary, binwidth = 0.01,
                        start = 0.01, end = 1.0) {
  iqr.breaks <- seq(from = start, to=end, by = binwidth)
  nbreaks <- length(iqr.breaks)
  iqr.labels <- iqr.breaks[2:nbreaks]
  iqr.tb <- table(cut(gene.summary$iqr,
                      breaks=iqr.breaks,
                      labels=iqr.labels))
  iqr.df <- data.frame(iqr = iqr.labels, freq = as.vector(iqr.tb))
  nrows <- nrow(iqr.df)
  iqr.df$csum <- cumsum(iqr.df$freq)
  iqr.df$slope <- (c(0, iqr.df$csum[2:nrows] - iqr.df$csum[1:(nrows - 1)])) /
      rep(binwidth, nrows)
  iqr.df$deriv <- - iqr.df$slope
  iqr.df
}

genes.iqr <- function(eqmatrix){
    gsm <- adply(eqmatrix, 1, quantile)
    gsm$iqr <- abs(gsm[,5] - gsm[,3])
    gsm
}

genes.iqr.plot <- function(gene.summary, start.limit = 2000,
                           max.limit = 23000){
    ggplot(gene.summary) +
        stat_bin(data = gene.summary, binwidth = 0.01,
                 aes(x = iqr, y = 22810 - cumsum(..count..)), geom = "step") +
        scale_x_continuous(limits = c(0,1)) +
        scale_y_continuous(limits = c(start.limit, max.limit)) +
        theme_bw()
}

tc.combo <- function(tissue.types, category.types){
    ldply(tissue.types, function(x) {
        ldply(category.types, function(y) {
            if(!(is.na(x) & is.na(y)) ) {
                c(Category = y, Tissue = x)
            }
        })
    })
}

tc.combo.inc <- function(tissue.types, tissue.types.all,
                         category.types){
    ldply(tissue.types, function(x) {
        if(x %in% tissue.types.all) {
            ldply(c(NA), function(y){
                c(Category = y, Tissue = x)
            })
        } else {
            ldply(category.types, function(y) {
                if(!(is.na(x) & is.na(y)) ) {
                    c(Category = y, Tissue = x)
                }
            })
        }
    })
}

columns.meta <- function(edata.colnames){
    ldply(1:length(edata.colnames), function(i){
        sname <- edata.colnames[i]
        db.name <- str_replace_all(unlist(str_split(sname, "_"))[1],
                                   "\\.", "-")
        db.name <- str_replace_all(db.name, "NASC-", "Exp")
        c(Column = i, Access = db.name)
    })
}

tc.access.names <- function(i, tissue.category.combo,
                            tissue.table, category.table){
  tissue <- tissue.category.combo[i, "Tissue"]
  category <- tissue.category.combo[i, "Category"]
  tissue.access.names <- if(!is.na(tissue)){
    tissue.table[ tissue.table$Tissue == tissue, "Access"]
  } else {
    unique(tissue.table$Access)
  }
  category.access.names <- if(!is.na(category)){
    category.table[ category.table$Category == category, "Access"]
  } else {
    unique(category.table$Access)
  }
  if(is.na(tissue)){
      category.access.names
  } else if(is.na(category)){
      tissue.access.names
  } else {
      intersect(category.access.names, tissue.access.names)
  }
}

cel.cols <- function(i, tissue.category.combo,
                     tissue.table, category.table,
                     edata.coldb){
    access.names <- tc.access.names(i, tissue.category.combo,
                                    tissue.table, category.table)
    edata.coldb[ edata.coldb$Access %in% access.names, "Column" ]
}

count.iqr.cols <- function(i, tissue.category.combo,
                           tissue.table, category.table,
                           edata.coldb){
    tissue <- tissue.category.combo[i, "Tissue"]
    category <- tissue.category.combo[i, "Category"]
    c(Category = category, Tissue = tissue,
      Access.Length = length(tc.access.names(i, tissue.category.combo,
          tissue.table, category.table)),
      CELs = length(cel.cols(i, tissue.category.combo,
          tissue.table, category.table, edata.coldb)))
}

tc.stats <- function(tissue.category.combo,
                     tissue.table, category.table,
                     edata.coldb){
    ldply(1:nrow(tissue.category.combo),
          function(i){
              count.iqr.cols(i, tissue.category.combo,
                             tissue.table, category.table,
                             edata.coldb);
          })
}

tc.iqr <- function(tissue.category.combo,
                   tissue.table, category.table,
                   edata.coldb, ematrix){
    llply(1:nrow(tissue.category.combo), function(i){
        scols <- as.integer(cel.cols(i, tissue.category.combo,
                                     tissue.table, category.table,
                                     edata.coldb))
        print(paste(i, length(scols)))
        if(length(scols) > 0) {
            qn_iqr_dry(ematrix, scols, 0.6)
        } else {
            NA
        }
    })
}

tc.iqr.auto <- function(tissue.category.combo,
                   tissue.table, category.table,
                   edata.coldb, ematrix){
    llply(1:nrow(tissue.category.combo), function(i){
        scols <- as.integer(cel.cols(i, tissue.category.combo,
                                     tissue.table, category.table,
                                     edata.coldb))
        print(paste(i, length(scols)))
        if(length(scols) > 0) {
            qn_iqr_dry_auto(ematrix, scols)
        } else {
            NA
        }
    })
}

tc.iqr.stats <- function(tissue.category.stats, tissue.category.iqr){
  tissue.category.stats$IQRFilter.Genes <- 0
  for(i in 1:nrow(tissue.category.stats)){
    tissue.category.stats$IQRFilter.Genes[i] <-
      if( length(tissue.category.iqr[[i]]) == 1 &&
         any(is.na(tissue.category.iqr[[i]]))) {
        0
      } else {
        length(tissue.category.iqr[[i]])
      }
  }
  tissue.category.stats
}

tc.iqr.stats.auto <- function(tissue.category.stats, tissue.category.iqr){
    tissue.category.stats$iqr_val <- 0.0
    for(i in 1:nrow(tissue.category.stats)){
        tissue.category.stats$iqr_val[i] <-
            if( length(tissue.category.iqr[[i]]) == 1 &&
               any(is.na(tissue.category.iqr[[i]]))) {
                NA
            } else {
                tissue.category.iqr[[i]]$iqr
            };
        tissue.category.stats$IQRFilter.Genes[i] <-
            if( length(tissue.category.iqr[[i]]) == 1 &&
               any(is.na(tissue.category.iqr[[i]]))) {
                0
            } else {
                length(tissue.category.iqr[[i]]$idx )
            }
    }
    tissue.category.stats
}

tc.iqr.genes <- function(tissue.category.iqr){
    ldply(1:length(tissue.category.iqr), function(x){
     if(!any(is.na(tissue.category.iqr[[x]]))){
        data.frame(combo = x, genes = tissue.category.iqr[[x]])
     } })
}

tc.iqr.genes.auto <- function(tissue.category.iqr){
    ldply(1:length(tissue.category.iqr), function(x){
        if(!any(is.na(tissue.category.iqr[[x]]))){
            data.frame(combo = x, genes = tissue.category.iqr[[x]]  $  idx)
        } })
}

tc.iqr.genes.plot <- function(tcat.iqry.genes){
  tcat.iqry.genes.summary = ddply(tcat.iqry.genes,
      .(genes), function(x){ c(combinations =  nrow(x)) })
  ggplot(tcat.iqry.genes.summary, aes(x = combinations)) + stat_bin(binwidth=1)
}

gen.combo.file <- function(i, Midx, ematrix, tissue.category.combo, tissue.table,
                           category.table, edata.coldb, spath, fname = ""){
    fname <- paste(spath, fname, ".csv", sep = "")
    scols <- as.integer(cel.cols(i, tissue.category.combo, tissue.table,
                                    category.table, edata.coldb))
    print(paste(fname, length(scols), length(Midx), sep = " "))
    rv <- if(any(is.na(Midx))){
        FALSE
    } else {
        M0 = normalizeBetweenArrays(ematrix[Midx, scols], method="quantile")
        write.table(file=fname, M0,
                    row.names=TRUE, col.names=TRUE,
                    sep="\t", quote=FALSE)
      TRUE
    }
    c(combo = i, columns = length(scols), rows = length(Midx),
      fname = fname, ret = rv)

}

gene.combo.file.cid <- function(i, ematrix, tissue.category.combo,
                                tissue.table,
                                category.table, edata.coldb, spath,
                                fname ){
    scols <- as.integer(cel.cols(i, tissue.category.combo, tissue.table,
                                    category.table, edata.coldb))
    Midx <- qn_iqr_dry_auto(ematrix, scols)
    gen.combo.file(i, Midx$idx, ematrix, tissue.category.combo,
                   tissue.table, category.table, edata.coldb, spath,
                   fname)
}

gen.combo.files <- function(tissue.category.stats, tissue.category.iqr,
                            ematrix, tissue.category.combo,
                            tissue.table, category.table, edata.coldb,
                            spath ){
    llply(1:nrow(tissue.category.stats), function(i) {
        Midx <- tissue.category.iqr[[i]]
        gen.combo.file(i, Midx, ematrix, tissue.category.combo,
                       tissue.table, category.table, edata.coldb,
                       spath, as.character(i))
    })
}

gen.combo.files.auto <- function(tissue.category.stats, tissue.category.iqr,
                            ematrix, tissue.category.combo,
                            tissue.table, category.table, edata.coldb,
                            spath ){
    llply(1:nrow(tissue.category.stats), function(i) {
        Midx <- tissue.category.iqr[[i]]$idx
        gen.combo.file(i, Midx, ematrix, tissue.category.combo,
                       tissue.table, category.table, edata.coldb,
                       spath)
    })
}

map.type.tables <- function(ttable, type.mapping, colname){
  # assumes the first column is the old; and the second column is the
  # new one
  alt.types <- unique(type.mapping[,2])
  ldply(alt.types, function(ct){
    types <- type.mapping[ type.mapping[,2] == ct , 1]
    rtable <- ttable[ ttable[, colname] %in% types, ]
    rtable[, colname] <- ct
    rtable
  })
}

#

make.iqr.plot <- function(iqr.df,
                          font_name = "Calibri",
                          font_size = 18) {
    ggplot(data = iqr.df,aes(x = iqr, y = freq)) +
        scale_x_continuous(limits = c(0.25,1.25)) +
        xlab("IQR") +
        ylab("Genes") +
        geom_point() +
        geom_line() +
        theme(text=element_text(family=font_name, size=font_size)) +
        theme_bw()
}

columns.iqr.bar <- function(iqr.val,
                            font_name = "Calibri",
                            font_size = 18,
                            bwidth = 0.02) {
    ggplot(data = data.frame(iqr = iqr.val), aes(x = iqr)) +
        stat_bin(binwidth = bwidth, geom = "line") +
        xlab("IQR") +
        ylab("Genes") +
        scale_x_continuous(limits = c(0.25,1.25)) +
        theme(text=element_text(family=font_name, size=font_size)) +
        theme_bw()
}

iqr.bar.plot <- function(iqr.df, cls.name, delta=-0.075, binwidth=0.05,
                         start=0.225, end=1.225, width=0.02){
    tx.cdf <-  iqr.df[iqr.df$class==cls.name, ]
    tx.df <- iqr.deriv.df(tx.cdf, binwidth=binwidth, start=start,
                          end=end)
    tx.df$iqr = tx.df$iqr + delta
    tx.xlabel = paste("IQR (± ", binwidth/2, ")", sep ="")
    ggplot(data=tx.df[tx.df$iqr > 0.38, ], aes(x=iqr, y=freq)) +
        geom_bar(stat='identity', width=width) +
        xlab(tx.xlabel) +
        ylab("No. of Genes")

}

iqr.line.plot <- function(iqr.df, cls.name, delta=-0.075, binwidth=0.05,
                         start=0.225, end=1.225){
    tx.cdf <-  iqr.df[iqr.df$class==cls.name, ]
    tx.df <- iqr.deriv.df(tx.cdf, binwidth=binwidth, start=start,
                          end=end)
    tx.df$iqr = tx.df$iqr + delta
    tx.xlabel = paste("IQR (± ", binwidth/2, ")", sep ="")
    ggplot(data=tx.df[tx.df$freq > 0, ], aes(x=iqr, y=freq)) +
        geom_point() + geom_line() +
        xlab(tx.xlabel) +
        ylab("No. of Genes")

}

probe.map.filter <- function(probe.ids, csv.names,
                             probe.out.names){
    cnames = read.table(csv.names, stringsAsFactors = F)$V1
    cnames = cnames[2:length(cnames)]
    pdf = probe.ids[cnames,]
    write.table(pdf, probe.out.names, quote = F, sep = "\t", col.names = F,
                row.names = F)
}
